import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;

import people.*;
import processing.core.*;
import processing.xml.XMLElement;
import ddf.minim.*;

public class InbredVis extends PApplet{

	boolean debug = false;
	boolean deploy = false;

	/*
	 * Data structures
	 */
	private Hashtable<String, RealNameProfile> realNameProfilesHash;
	private ArrayList<RealNameProfile> realNameProfiles;
	private Hashtable<String, MainAlias> mainAliasesHash;
	private ArrayList<MainAlias> mainAliases;
	private ArrayList<MainAliasCompByGroups> mainAliasesByGroup;
	private Hashtable<String, Alias> aliasesHash; //<artist real name, 
	private ArrayList<Alias> aliases;
	private Hashtable<String, Group> groupsHash;
	private ArrayList<Group> groups;

	/*
	 * Other Vars
	 */
	PFont font;
	float maxFontSize = 30;
	float fontSizeRatio = 1;
	int maxNumTracks;
	int pieChartTotalTracks = 0;

	public static void main(String args[]) {
		//PApplet.main(new String[] { "--present", "InbredVis" });
	}


	AudioPlayer player;
	Minim minim;
	public void setup() {
		minim = new Minim(this);
		if (deploy){
			player = minim.loadFile("RockThisParty.mp3", 2048);
		} else {
			player = minim.loadFile("C:\\Documents and Settings\\Empy\\My Documents\\Processing\\jsinclai_Project1\\data\\RockThisParty.mp3", 2048);
		}
		// play the file
		player.play();

		System.out.println(color(200));
		//size(200,200);
		background(255);
		size(1000,600);
		mainAliases = new ArrayList<MainAlias>();
		mainAliasesByGroup = new ArrayList<MainAliasCompByGroups>();
		mainAliasesHash = new Hashtable<String, MainAlias>();
		groupsHash = new Hashtable<String, Group>();
		aliases = new ArrayList<Alias>();
		aliasesHash = new Hashtable<String, Alias>();
		realNameProfilesHash = new Hashtable<String, RealNameProfile>();
		realNameProfiles = new ArrayList<RealNameProfile>();
		groups = new ArrayList<Group>();

		//String[] fontList = PFont.list();
		//println(fontList);
		font = createFont("Trebuchet MS", 32);
		// Set the font and its size (in units of pixels)
		textFont(font, 32);

		text("Loading...", width/2, height/2);

		loadData();

		if (debug) noLoop();

		for (int i = 0; i < mainAliases.size(); i++){
			MainAlias ma = mainAliases.get(i);
			pieChartTotalTracks += ma.getTotalArtistReleases();
		}
	}

	public void stop(){
		player.close();
		minim.stop();

		super.stop();
	}

	private void loadData() {
		//Read in XML files
		String lines[];
		if (deploy){
			lines = loadStrings("listing.txt");
		} else {
			lines = loadStrings("C:\\Documents and Settings\\Empy\\My Documents\\Processing\\jsinclai_Project1\\data\\listing.txt");
		}
		//
		//println("there are " + lines.length + " lines");
		for (int i=0; i < lines.length; i++) {
			//println(lines[i]);
			importData(lines[i]); 
		}

		connectData();
	}

	private void connectData() {
		/*
		 * Need to check if an alias is a main alias.
		 * 
		 * Get all the main aliases in a list
		 */
		for (int i = 0; i < realNameProfiles.size(); i++){
			RealNameProfile rnp = realNameProfiles.get(i);
			String mainAliasName = rnp.getMainAliasName();

			if (aliasesHash.containsKey(mainAliasName)){
				Alias a = aliasesHash.get(mainAliasName);
				//this is supposed to be a MainAlias. turn it into one.

				MainAlias ma = new MainAlias(this, rnp, a.getOtherAliasNames(), a.getNumReleases());
				MainAliasCompByGroups ma2 = new MainAliasCompByGroups(this, rnp, a.getOtherAliasNames(), a.getNumReleases());
				mainAliases.add(ma);
				mainAliasesByGroup.add(ma2);
				aliasesHash.remove(mainAliasName);
			}
		}

		/*
		 * For each main alias:
		 * 1. Add their other aliases
		 * 2. add their groups
		 */
		//add other aliases
		for (int i = 0; i < mainAliases.size(); i++){
			MainAlias ma = mainAliases.get(i);
			MainAliasCompByGroups ma2 = mainAliasesByGroup.get(i);
			String[] otherAliases = ma.getOtherAliasesNames();

			for (int j = 0; j < otherAliases.length; j++){
				String aliasName = otherAliases[j];
				Alias a = aliasesHash.get(aliasName);
				ma.addOtherAlias(a);
				ma2.addOtherAlias(a);
			}

			String[] groupNames = ma.getRealNameProfile().getGroupNames();
			if (groupNames != null){
				//some artists are in no groups. e.g. Dj Stompy

				for (int j = 0; j < groupNames.length; j++){
					Group g = groupsHash.get(groupNames[j]);
					ma.addGroup(g);
					ma2.addGroup(g);
				}
			}

			ma.setTotalArtistReleases(ma.getTotalArtistReleases());
			ma2.setTotalGroupReleases(ma2.getTotalGroupsReleases());

			if (debug) System.out.println(ma);
		}

		//now order the data
		Collections.sort(groups);
		Collections.sort(mainAliases);
		Collections.sort(mainAliasesByGroup);


		//TODO: fix this
		//set the maxNumber of tracks (by either artist or group
		maxNumTracks = mainAliases.get(0).getTotalArtistReleases() > groups.get(0).getNumReleases() ? 
				mainAliases.get(0).getTotalArtistReleases() : groups.get(0).getNumReleases();
	}

	public void draw() {
		background(255);

		//drawGraphChart();
		drawVerticalChart1();


		//arc(x, y, width, height, start, stop)
		/*
		int diameter = 200;
		float lastAng = 0;
		for (int i = 0; i < mainAliases.size(); i++){
			MainAlias ma = mainAliases.get(i);
			float proportion = (float)ma.getTotalArtistReleases()/(float)pieChartTotalTracks;
			fill((float)(proportion*3.0));
			arc (width/2, height/2, diameter, diameter, lastAng, lastAng+radians(proportion));
			lastAng += radians(proportion);
		}

		int diameter = 150;
		int[] angs = {30, 10, 45, 35, 60, 38, 75, 67};
		float lastAng = 0;

		for (int i = 0; i < angs.length; i++){
		  fill(angs[i] * 3.0);
		  arc(width/2, height/2, diameter, diameter, lastAng, lastAng+radians(angs[i]));
		  lastAng += radians(angs[i]);  
		 */
	}

	private void drawGraphChart() {
		//Draw the graph
		stroke(0);
		line(100, 100, 100, 500);
		line(100, 500, 700, 500);

		for (int i = 0; i < groups.size(); i++){
			Group g = groups.get(i);
			float gX = calculateX(g.getNumReleases(), maxNumTracks);
			float gY = calculateY(g.getNumAliases());
			System.out.println(gX + "/" + gY);

			g.drawChartPosition(gX, gY, font);
		}

		for (int i = 0; i < mainAliases.size(); i++){
			MainAlias ma = mainAliases.get(i);
			float gX = calculateX(ma.getTotalArtistReleases(), maxNumTracks);
			float gY = calculateY(ma.getNumAliases());
			System.out.println(gX + "/" + gY);

			ma.drawChartPosition(gX, gY, font);
		}
	}

	private float calculateY(int length) {
		int yOffset = 100;
		int yInterval = 400;
		float intervalSpacing = 400/15; //400 px / 15 intervals

		//y of 100 is top
		//y of 400 is bottom
		// if length == 0, then should return 400

		return (yOffset + yInterval) - (length * intervalSpacing);
	}

	private float calculateX(double numReleases, double maxNumTracks2) {

		//float x = (float) (Math.log(numReleases * 1.5 + 1.0) * 20);

		double x = 100 + (600.00*(numReleases/maxNumTracks2));
		System.out.println(x+ "/" + new Float(x));
		return new Float(x);
	}

	private void drawVerticalChart1() {
		//draw labels
		fill(0);
		textFont(font, 20);
		textAlign(CENTER);
		text("Solo releases", width/4, 25);
		text("Groups sorted by releases", width/2, 25);
		text("Group Releases", width*3/4, 25);


		float yOffset = 40;
		int columnXOffset = 0;
		int counter = 1; //counts how many columns we've made
		for (int i = 0; i < groups.size(); i++){
			Group g = groups.get(i);

			float height = g.draw(width*7/16-50+columnXOffset, yOffset, font);

			yOffset+= 30 + height;
			if (yOffset > 550){
				columnXOffset += 35;
				yOffset = 40 + (counter*5);
				counter++;
			}
		}

		yOffset = 40;
		columnXOffset = 0;
		counter = 1;
		//draw the MainAliases
		//we assume that the groups have been drawn at this point
		// so we can connect the MAs to their groups
		for (int i = 0; i < mainAliases.size(); i++){
			MainAlias ma = mainAliases.get(i);
			float height = ma.drawRect(width/4- 50+columnXOffset, yOffset);
			ma.connectToGroups();

			if (i == 0){ fontSizeRatio = maxFontSize/height; }
			ma.drawFont(font, fontSizeRatio, maxFontSize);

			yOffset+= 10 + height;
			if (yOffset > 550){
				columnXOffset += 80;
				yOffset = 40 + (counter*5);
				counter++;
			}
		}

		//do the same work for the MA-by-group
		yOffset = 40;
		columnXOffset = 0;
		counter = 1;
		for (int i = 0; i < mainAliasesByGroup.size(); i++){
			MainAliasCompByGroups ma = mainAliasesByGroup.get(i);
			float height = ma.drawRect(width*3/4 - 50+columnXOffset, yOffset);
			ma.connectToGroups();

			if (i == 0){ fontSizeRatio = maxFontSize/height; }
			ma.drawFont(font, fontSizeRatio, maxFontSize);

			yOffset+= 10 + height;
			if (yOffset > 550){
				columnXOffset += 80;
				yOffset = 40 + (counter*5);
				counter++;
			}
		}

		for (int i = 0; i <mainAliases.size(); i++){
			MainAlias ma = mainAliases.get(i);
			if (ma.isMouseOver()){
				for (int j=0; j < ma.groups.size(); j++){
					Group g = ma.groups.get(j);
					if (g!=null){
						g.setSelected();
					}
				}
			}

			MainAliasCompByGroups ma2 = mainAliasesByGroup.get(i);
			if (ma2.isMouseOver()){
				for (int j=0; j < ma2.groups.size(); j++){
					Group g = ma2.groups.get(j);
					if (g!=null){
						g.setSelected();
					}
				}
			}
		}

		for (int i = 0; i < groups.size(); i++){
			Group g = groups.get(i);
			g.drawFont(g.xPosition, g.yPosition, font);
		}
	}

	/*
	 * TODO: fix this
	 */
	public void mousePressed(){
		/*
		println("press");
		for (int i = 0; i < mainAliases.size(); i++){
			MainAlias ma = mainAliases.get(i);
			if (ma.isMouseOver()){
				ma.setClicked();
			} else {
				ma.setUnclicked();
			}
		}
		 */
	}


	private void importData(String filename){
		String artistName = null;
		String artistRealName = null;
		String mainAlias = null;
		String[] otherAliasNames = null;
		String[] inGroups = null;
		String[] members = null;
		int releases = 0;

		XMLElement xml;
		if (deploy){
			xml = new XMLElement(this, filename);
		} else {
			xml = new XMLElement(this, "C:\\Documents and Settings\\Empy\\My Documents\\Processing\\jsinclai_Project1\\data\\"+filename);

		}
		XMLElement artist = xml.getChild(0);
		int elementCount = artist.getChildCount();

		for (int k = 0; k < elementCount; k++){
			XMLElement artistKid = artist.getChild(k);
			if(artistKid.getName().compareTo("name") == 0)
			{
				artistName = artistKid.getContent();
			} 
			else if (artistKid.getName().compareTo("realname") == 0)
			{
				artistRealName = artistKid.getContent();
			} 
			else if (artistKid.getName().compareTo("aliases") == 0)
			{
				otherAliasNames = new String[artistKid.getChildCount()];
				for(int i = 0; i < artistKid.getChildCount(); i++){
					otherAliasNames[i] = artistKid.getChild(i).getContent();
				}
			} 
			else if (artistKid.getName().compareTo("mainalias") == 0)
			{
				mainAlias = artistKid.getContent();
			} 
			else if (artistKid.getName().compareTo("groups") == 0)
			{
				inGroups = new String[artistKid.getChildCount()];
				for(int i = 0; i < artistKid.getChildCount(); i++){
					inGroups[i] = artistKid.getChild(i).getContent();
				}
			} 
			else if (artistKid.getName().compareTo("members") == 0)
			{
				// then this is a group with multiple artists
				members = new String[artistKid.getChildCount()];
				for(int i = 0; i < artistKid.getChildCount(); i++){
					members[i] = artistKid.getChild(i).getContent();
				}
			} 
			else if (artistKid.getName().compareTo("releases") == 0)
			{
				releases = artistKid.getChildCount();
			} 
		}

		if (members != null){
			Group g = new Group(this, artistName, members, releases, otherAliasNames);
			//System.out.println("Group: " + g);
			groupsHash.put(artistName, g);
			groups.add(g);
		} else {
			if (mainAlias != null){
				//then this is an Artist's Real Name Profile
				RealNameProfile rnp = new RealNameProfile(artistName, mainAlias, inGroups);
				realNameProfilesHash.put(artistName, rnp);
				realNameProfiles.add(rnp);

				if (artistName.compareTo(mainAlias) == 0){
					//then this is ALSO the mainAlias. So build an alias for it
					//This is a special case, like for Scott Brown
					Alias a = new Alias(this, artistName, artistRealName, otherAliasNames, releases);
					aliases.add(a);
					aliasesHash.put(artistName, a);
				}
			} 
			else {
				//this is an Alias
				Alias a = new Alias(this, artistName, artistRealName, otherAliasNames, releases);
				aliases.add(a);
				aliasesHash.put(artistName, a);
			}
		}
	}
}
