package people;

import processing.core.PApplet;
import processing.core.PFont;

public class Group implements Comparable<Group>{
	PApplet p;
	String groupName;
	String[] members;
	String[] aliases;
	int releases;

	//Drawing and shape variables
	public float yPosition;
	public float xPosition;
	public float shapeSize;

	//vertical color variables
	int rectColor = 0;
	float rectWeight = 0;
	float fontSize = 10;
	boolean isSelected = false;
	
	//chart color variables
	int chartRectFillColor;
	int chartRectStrokeColor = 255;
	int chartRectStrokeWeight = 0;

	public Group(PApplet p, String groupName, String[] members, int releases, String[] aliases){
		this.p = p;

		this.groupName = groupName;
		this.members = members;
		this.aliases = aliases;
		this.releases = releases;
		
		//init colors
		chartRectFillColor = p.color(146, 245, 204);
	} 

	public void drawChartPosition(float gX, float gY, PFont font) {
		xPosition = gX;
		yPosition = gY;
		shapeSize = 12;
		
		if (isMouseOver()){
			System.out.println("Mouse over");
			setColors();
			setFont();
		} else {
			System.out.println("No Mouse over");
			unsetFont();
			unsetColors();
		}
		
		p.fill(chartRectFillColor);
		p.stroke(chartRectStrokeColor);
		if (chartRectStrokeWeight == 0){p.noStroke();} else { p.strokeWeight(chartRectStrokeWeight);}
		p.rect(gX, gY, shapeSize, shapeSize);
	}

	private void unsetColors() {
		//revert back to default colors
		chartRectFillColor = p.color(146, 245, 204);;
		chartRectStrokeColor = 255;
		chartRectStrokeWeight = 0;
	}

	private void unsetFont() {
		// TODO Auto-generated method stub
	}

	private void setFont() {
		// TODO Auto-generated method stub
		
	}

	private void setColors() {
		chartRectFillColor = p.color(255,127,0);
		chartRectStrokeColor = 0;
		chartRectStrokeWeight = 1;
	}

	public float draw(float xOffset, float yOffset, PFont font) {

		//double size = Math.log(getNumReleases())*Math.log(getNumReleases());
		double size = Math.log(getNumReleases());
		float fSize = (float)size;
		//		System.out.println(getGroupName() + ": " + getNumReleases() + ", " + fSize);

		p.fill(0);
		p.stroke(rectColor);
		p.strokeWeight(rectWeight);
		//p.rect(xOffset, yOffset, fSize, fSize);
		p.ellipse(xOffset, yOffset, fSize, fSize);
		
		//drawFont(xOffset, yOffset, font);

		//store the position and size of this shape
		xPosition = xOffset;
		yPosition = yOffset;
		shapeSize = fSize;

		return fSize;
	}

	public void drawFont(float xOffset, float yOffset, PFont font) {
		float fSize = (float)Math.log(getNumReleases());
		
		if (isMouseOver() || isSelected){
			fontSize = 25;
			System.out.println(groupName + " selected");
		} else {
			fontSize = fSize;
		}
		p.fill(229, 25, 229);
		p.textFont(font, fontSize);
		p.textAlign(p.CENTER);
		p.text(getGroupName(), xOffset, yOffset+fSize-(fSize/10));
	}

	public boolean isMouseOver(){
		if (p.mouseX >= xPosition-10 && p.mouseX <= xPosition+shapeSize+10 && 
				p.mouseY >= yPosition-10 && p.mouseY <= yPosition+shapeSize+10) {
			//			setSelected();
			return true;
		} else {
			//			setDeselected();
			return false;
		}
	}

	public int getNumReleases(){
		return releases;
	}

	public String getGroupName() {
		return groupName;
	}

	public String[] getMembers() {
		return members;
	}

	public String[] getAliases() {
		return aliases;
	}

	public String toString(){
		return groupName;
	}

	@Override
	public int compareTo(Group o) {
		return  o.releases - this.releases;
	}

	public int getNumAliases() {

		if (aliases != null)
			return aliases.length;
		else
			return 0;
	}

	public void setSelected(){
		isSelected = true;
	}
	
	public void deselect(){
		isSelected = false;
	}


}
