package people;

import java.util.ArrayList;

import processing.core.PApplet;
import processing.core.PFont;

public class MainAlias implements Comparable<MainAlias>{

	PApplet p;
	RealNameProfile realName;
	String aliasName;
	String[] otherAliasesNames;
	int numReleases;
	int totalArtistReleases;
	ArrayList<Alias> aliases;
	public ArrayList<Group> groups;

	//A scrape string to figure out which aliases/groups are missing.
	String artistsMissing = "";

	//Drawing and shape variables
	float yPosition;
	float xPosition;
	float shapeSize;

	//color variables for hover and such
	//int lineColor; final int defLineColor = -8487298; //equiv of p.color(126);
	int lineColor; int defLineColor; //equiv of p.color(200);
	float lineWeight; final float defLineWeight = 1;
	int rectStrokeColor = 0;
	float rectWeight = 0;
	int rectFillColor = 0;
	float fontSize = 10;
	
	//color vars for the chart graph thing.
	int chartRectFillColor;
	int chartRectStrokeColor = 255;
	int chartRectStrokeWeight = 0;

	boolean isClicked = false;
	boolean mousedOver = false;

	public MainAlias(PApplet p, RealNameProfile realName, String[] otherAliasesNames, int numReleases){
		this.p = p;
		this.realName = realName;
		this.numReleases = numReleases;
		this.otherAliasesNames = otherAliasesNames;
		aliasName = realName.getMainAliasName();	

		defLineColor = p.color(255,0,0,30);
		
		aliases = new ArrayList<Alias>();
		groups = new ArrayList<Group>();
	}
	
	public void drawChartPosition(float gX, float gY, PFont font) {
		xPosition = gX;
		yPosition = gY;
		shapeSize = 12;
		
		if (isMouseOver()){
			System.out.println("Mouse over");
			setColors();
			setFont();
		} else {
			System.out.println("No Mouse over");
			unsetFont();
			unsetColors();
		}
		
		p.fill(chartRectFillColor);
		p.stroke(chartRectStrokeColor);
		if (chartRectStrokeWeight == 0){p.noStroke();} else { p.strokeWeight(chartRectStrokeWeight);}
		//p.rect(gX, gY, shapeSize, shapeSize);
		
	}

	private void unsetColors() {
		//revert back to default colors
		chartRectFillColor = p.color(255, 54, 252);
		chartRectStrokeColor = 255;
		chartRectStrokeWeight = 0;
	}

	private void unsetFont() {
		// TODO Auto-generated method stub
	}

	private void setFont() {
		// TODO Auto-generated method stub
		
	}

	private void setColors() {
		chartRectFillColor = p.color(255,127,0);
		chartRectStrokeColor = 0;
		chartRectStrokeWeight = 1;
	}

	public float drawRect(float xOffset, float yOffset) {

		double size = Math.log(totalArtistReleases)*Math.log(totalArtistReleases);
		float fSize = (float)size;

		//draw the shape representing this artist
		//TODO: split this up to include multiple aliases
		p.fill(rectFillColor);
		p.stroke(rectStrokeColor);
		p.strokeWeight(rectWeight);
		//p.rect(xOffset, yOffset, fSize, fSize);
		p.ellipseMode(p.CORNER);
		p.ellipse(xOffset, yOffset, fSize, fSize);
		//store the position and size of this shape
		xPosition = xOffset;
		yPosition = yOffset;
		shapeSize = fSize;

		return fSize;
	}
	
	public void drawFont(PFont font, float fontRatio, float maxFontSize){
		if (isClicked || isMouseOver()){
			fontSize = maxFontSize + 5;
		} else {
			fontSize = fontRatio*shapeSize;
		}
		
		//draw the text representing this artist
		p.fill(255, 14, 0);
		p.textFont(font, fontSize);
		p.textAlign(p.RIGHT);
		p.text(getMainAliasName(), xPosition+shapeSize, yPosition+shapeSize-(shapeSize/10));
	}

	public void connectToGroups() {
		if (isMouseOver())
			setMouseOver();
		else
			setMouseOut();

		for (int i = 0; i < groups.size(); i++){
			Group g = groups.get(i);
			//line(x1, y1, x2, y2)
			if (g != null){
				p.stroke(lineColor);
				p.strokeWeight(lineWeight);
				p.line(xPosition+(shapeSize/2), yPosition+(shapeSize/2), 
						g.xPosition+(g.shapeSize/2), g.yPosition+(g.shapeSize/2));
			}
		}
	}

	public void resetColors(){
		lineColor = defLineColor;
		lineWeight = defLineWeight;
		rectStrokeColor = 0;
		rectWeight = 0;
	}

	private void setMouseOver(){		
		lineColor = p.color(255, 50, 0);
		lineWeight = 2;
		rectStrokeColor = p.color(183, 95, 12);
		rectWeight = 2;
		for (int i = 0; i < groups.size(); i++){
			if (groups.get(i) != null){
				groups.get(i).setSelected();
			}
		}
	
	}

	private void setMouseOut(){
		/*
		 * When mousing out, check to see if the object is already clicked on.
		 * if it's clicked on, the return to the "clicked" state colors.
		 * otherwise eliminate all coloring and return to default state.
		 */
		if (isClicked){
			setClicked();
		} else {
			resetColors();
			for (int i = 0; i < groups.size(); i++){
				if (groups.get(i) != null){
					groups.get(i).deselect();
				}
			}
		}
	}

	public void setClicked() {
		isClicked = true;
		//set the color and size of the 
		// lines to groups
		lineColor = p.color(37, 15, 247);
		lineWeight = 4;

		//set the color and size of the 
		// rect stroke
		rectStrokeColor = p.color(25, 9, 178);
		rectWeight = 2;
	}

	public void setUnclicked() {
		isClicked = false;
		resetColors();
	}

	public boolean isMouseOver(){
		if (p.mouseX >= xPosition && p.mouseX <= xPosition+shapeSize && 
				p.mouseY >= yPosition && p.mouseY <= yPosition+shapeSize) {
			mousedOver = true;
			return true;
		} else {
			mousedOver = false;
			return false;
		}
	}

	@Override
	public int compareTo(MainAlias ma) {
		return ma.getTotalArtistReleases() - this.getTotalArtistReleases();
	}

	public void addOtherAlias(Alias a){
		aliases.add(a);
	}

	public void addGroup(Group g){
		groups.add(g);
	}

	public String getArtistRealName(){
		return realName.getRealName();
	}

	public String getMainAliasName(){
		return realName.getMainAliasName();
	}

	public RealNameProfile getRealNameProfile(){
		return realName;
	}

	public String[] getOtherAliasesNames(){
		return otherAliasesNames;
	}

	public int getNumReleases(){
		return numReleases;
	}

	public int getTotalArtistReleases(){
		int totalReleases = numReleases;
		for (Alias a : aliases){
			if (a != null)
				totalReleases += a.getNumReleases();
		}
		return totalReleases;
	}

	public String toString(){
		return "Main Alias: " + aliasName + 
		"\n--Other Aliases: "+ revealAliases(aliases, otherAliasesNames) +
		"\n--In Groups: " + revealGroups(groups, realName.getGroupNames());
	}

	private String revealAliases(ArrayList<Alias> existingAliases, String[] aliasNames){
		StringBuilder sb = new StringBuilder();
		StringBuilder sbMissing = new StringBuilder();
		for (int i = 0; i < existingAliases.size(); i++){
			if (existingAliases.get(i) == null){
				sb.append("[--"+aliasNames[i]+"--], ");
				sbMissing.append("\""+aliasNames[i]+"\",");
			} else {
				sb.append(existingAliases.get(i) + ", ");
			}
		}
		System.out.println("MISSING ARTISTS:\n" + sbMissing.toString().replace(" ", "+"));
		artistsMissing+=sbMissing.toString().replace(" ", "+");
		return sb.toString();
	}

	private String revealGroups(ArrayList<Group> existingGroups, String[] groupNames){
		StringBuilder sb = new StringBuilder();
		StringBuilder sbMissing = new StringBuilder();
		for (int i = 0; i < existingGroups.size(); i++){
			if (existingGroups.get(i) == null){
				sb.append("[--"+groupNames[i]+"--], ");
				sbMissing.append("\""+groupNames[i]+"\",");
			} else {
				sb.append(existingGroups.get(i) + ", ");
			}
		}
		System.out.println("MISSING GROUPS:\n" + sbMissing.toString().replace(" ", "+"));
		artistsMissing+=sbMissing.toString().replace(" ", "+");
		return sb.toString();
	}

	public int getNumAliases() {

		if (aliases != null)
			return aliases.size();
		else
			return 0;
	}

	public void setTotalArtistReleases(int totalArtistReleases2) {
		totalArtistReleases = totalArtistReleases2;
	}

}
